<?php

namespace Drupal\advanced_email_validation\Plugin\Validation\Constraint;

use Drupal\advanced_email_validation\AdvancedEmailValidator;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use EmailValidator\EmailValidator;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Validates the AdvancedEmailValidation constraint.
 */
class AdvancedEmailValidationValidator extends ConstraintValidator implements ContainerInjectionInterface {

  /**
   * Constructs a new AdvancedEmailValidationValidator object.
   *
   * @param \Drupal\advanced_email_validation\AdvancedEmailValidator $emailValidator
   *   The email validator service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   */
  public function __construct(protected AdvancedEmailValidator $emailValidator, protected ConfigFactoryInterface $configFactory) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('advanced_email_validation.validator'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function validate($value, Constraint $constraint): void {
    $config = $this->configFactory->get('advanced_email_validation.settings');
    $validateAccountOn = $config->get('validate_account_on');
    $isNew = $value->getParent()->getEntity()->isNew();

    if ($isNew && !$validateAccountOn['created']) {
      return;
    }
    if (!$isNew && !$validateAccountOn['updated']) {
      return;
    }

    $email = $value->getString();
    if (empty($email)) {
      return;
    }
    $result = $this->emailValidator->validate($email);

    if ($result !== EmailValidator::NO_ERROR) {
      $errorMessage = $this->emailValidator->errorMessageFromCode($result);
      if (empty($errorMessage)) {
        $this->context->addViolation($constraint->defaultError);
      }
      $this->context->addViolation($errorMessage);
    }
  }

}
